# Generated by Django 3.2 on 2021-10-27 14:03

import django.contrib.postgres.search
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import urlid_graph.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ObjectRepository',
            fields=[
                ('uuid', models.UUIDField(primary_key=True, serialize=False)),
                ('internal_id', models.TextField()),
                ('search_data', django.contrib.postgres.search.SearchVectorField(null=True)),
            ],
            options={
                'db_table': 'urlid_object_repository',
                'managed': False,
            },
            bases=(urlid_graph.models.ObjectMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Entity',
            fields=[
                ('uuid', models.UUIDField(primary_key=True, serialize=False)),
                ('base_url', models.TextField()),
                ('name', models.TextField()),
                ('version', models.TextField()),
            ],
            managers=[
                ('brasilio', django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name='Property',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField()),
                ('value', models.TextField(blank=True, null=True)),
                ('value_type', models.SmallIntegerField(choices=[(1, 'str'), (2, 'date'), (3, 'datetime'), (4, 'float'), (5, 'int'), (6, 'bool')])),
                ('source', models.TextField()),
                ('value_datetime', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(blank=True, null=True)),
                ('object', models.ForeignKey(db_column='object_uuid', db_constraint=False, db_index=False, on_delete=django.db.models.deletion.DO_NOTHING, to='urlid_graph.objectrepository')),
            ],
        ),
        migrations.CreateModel(
            name='Object',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(db_index=True)),
                ('internal_id', models.TextField()),
                ('entity', models.ForeignKey(db_column='entity_uuid', db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='urlid_graph.entity')),
            ],
            bases=(urlid_graph.models.ObjectMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ElementConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('config_type', models.PositiveSmallIntegerField(choices=[(1, 'entity'), (2, 'relationship'), (3, 'property'), (4, 'stats')])),
                ('name', models.TextField()),
                ('parent_type', models.PositiveSmallIntegerField(choices=[(1, 'entity'), (2, 'relationship')], null=True)),
                ('parent_name', models.TextField(null=True)),
                ('label', models.TextField()),
                ('data', models.JSONField()),
            ],
            options={
                'unique_together': {('config_type', 'name', 'parent_type', 'parent_name')},
            },
        ),
        migrations.CreateModel(
            name='EntityConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('urlid_graph.elementconfig',),
        ),
        migrations.CreateModel(
            name='EntityPropertyConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('urlid_graph.elementconfig',),
        ),
        migrations.CreateModel(
            name='RelationshipConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('urlid_graph.elementconfig',),
        ),
        migrations.CreateModel(
            name='RelPropertyConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('urlid_graph.elementconfig',),
        ),
        migrations.CreateModel(
            name='StatsConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('urlid_graph.elementconfig',),
        ),
        migrations.AddIndex(
            model_name='property',
            index=models.Index(fields=['object', 'value_type'], name='urlid_graph_object__fbf252_idx'),
        ),
    ]



import csv
import random

from django.core.management.base import BaseCommand
from model_bakery.baker import random_gen


class Command(BaseCommand):
    help = "Create fake relations"

    def add_arguments(self, parser):
        parser.add_argument("output_filename")
        parser.add_argument("number_rows", type=int)

    def gen_row(self):
        return {
            "from_node_uuid": random_gen.gen_uuid(),
            "to_node_uuid": random_gen.gen_uuid(),
            "inicio": random_gen.gen_date(),
            "fim": random_gen.gen_date() if random.random() < 0.9 else "",
            "descricao": random_gen.gen_text() if random.random() < 0.9 else "",
            "quantidade": random_gen.gen_integer() if random.random() < 0.9 else "",
            "valor": random_gen.gen_integer() if random.random() < 0.9 else "",
        }

    def handle(self, *args, **options):
        with open(options["output_filename"], mode="w", newline="") as csvfile:
            fieldnames = [
                "from_node_uuid",
                "to_node_uuid",
                "inicio",
                "fim",
                "descricao",
                "quantidade",
                "valor",
            ]
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()

            for _ in range(options["number_rows"]):
                writer.writerow(self.gen_row())



# Generated by Django 3.2 on 2021-11-01 18:44

from django.db import migrations


MIGRATION_SQL = [
    """
    CREATE MATERIALIZED VIEW urlid_graph_object_fts AS
        SELECT
            object_uuid,
            to_tsvector('pg_catalog.portuguese', STRING_AGG(value, ' ')) AS search_data
        FROM urlid_graph_property
        WHERE
            value_type = 1
            AND value IS NOT NULL
            AND value <> ''
        GROUP BY object_uuid
        WITH NO DATA;
    """,
    """
    CREATE UNIQUE INDEX IF NOT EXISTS idx_urlid_object_fts_uuid ON urlid_graph_object_fts (
        object_uuid
    );
    """,
    """
    CREATE INDEX IF NOT EXISTS idx_urlid_object_fts_vector ON urlid_graph_object_fts USING gin (
        search_data
    );
    """,
]


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0002_savedgraph'),
    ]

    operations = [
        migrations.RunSQL(sql=MIGRATION_SQL),
    ]



import traceback

import rows
from django.core.management.base import BaseCommand
from django.db import connections

from .models import get_urlid_database_uri
from .utils import DatabaseConnection, read_total_size, working
import urlid_graph.settings as urlid_graph_settings


class ImportCommand(BaseCommand):
    help = "Import rows into table"
    schema = None

    def add_arguments(self, parser):
        parser.add_argument("-b", "--batch-size", type=int, default=8388608)
        parser.add_argument("input_filename", nargs="+")

    def import_data(self, input_filename, batch_size):
        rows_progress = rows.utils.ProgressBar(
            prefix="Importing data",
            pre_prefix="Detecting file size",
            unit="bytes",
            total=read_total_size(input_filename),
        )
        try:
            rows_output = rows.utils.pgimport(
                input_filename,
                encoding="utf-8",
                dialect="excel",
                table_name=self.table_name,
                database_uri=get_urlid_database_uri(),
                schema=self.schema,
                create_table=False,
                chunk_size=batch_size,
                callback=rows_progress.update,
            )
        except RuntimeError:
            rows_progress.close()
            # TODO: process the exception and show a good error message
            raise
        else:
            rows_progress.description = f"Imported {rows_output['rows_imported']} rows to '{self.table_name}'"
            rows_progress.close()

    def handle(self, *args, **options):
        input_filenames = options["input_filename"]
        batch_size = options["batch_size"]
        connection = connections[urlid_graph_settings.DJANGO_DATABASE]
        db = DatabaseConnection(connection=connection)
        ok = True

        try:
            with working("Disabling sync commit"):
                db.disable_sync_commit()
            with working(f"Disabling autovacuum for {self.table_name}"):
                db.disable_autovacuum(self.table_name)
            with working(f"Disabling indexes on {self.table_name}"):
                db.execute_query(
                    f"UPDATE pg_index SET indisready = FALSE WHERE indrelid = '{self.table_name}'::regclass"
                )
            with working("Disabling triggers"):
                db.execute_query(f"ALTER TABLE {self.table_name} DISABLE TRIGGER ALL")

            for filename in input_filenames:
                self.import_data(filename, batch_size)

        except:  # noqa
            traceback.print_exc()
            ok = False

        finally:
            with working(f"Reindexing table {self.table_name}"):
                db.execute_query(f"REINDEX TABLE {self.table_name}")
            with working(f"Reenabling triggers on {self.table_name}"):
                db.execute_query(f"ALTER TABLE {self.table_name} ENABLE TRIGGER ALL")
            with working(f"Running VACUUM ANALYZE on {self.table_name}"):
                db.vacuum_analyze(self.table_name)
            with working(f"Enabling autovacuum for {self.table_name}"):
                db.enable_autovacuum(self.table_name)
            with working("Enabling sync commit"):
                db.enable_sync_commit()

        return str(ok)  # Used by import_data when calling this command programatically



# Generated by Django 3.2 on 2021-11-01 18:48

from django.db import migrations


CREATE_VIEW_SQL = """
    CREATE VIEW urlid_graph_object_repository AS
        SELECT
            o.uuid,
            o.entity_uuid,
            o.internal_id,
            s.search_data
        FROM urlid_graph_object AS o
          LEFT JOIN urlid_graph_object_fts AS s
            ON s.object_uuid = o.uuid;
"""

DROP_VIEW_SQL = """
    DROP VIEW IF EXISTS urlid_graph_object_repository;
"""


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0003_auto_20211101_1844'),
    ]

    operations = [migrations.RunSQL(sql=CREATE_VIEW_SQL, reverse_sql=DROP_VIEW_SQL)]



# Generated by Django 3.2 on 2021-11-04 12:45

from django.db import migrations

CREATE_VLABEL_OBJECT = "CREATE VLABEL IF NOT EXISTS object;"
DROP_VLABEL_OBJECT = "DROP VLABEL IF EXISTS object;"

CREATE_PROPERTY_INDEX = "CREATE PROPERTY INDEX IF NOT EXISTS idx_vlabel_object_uuid ON object (uuid);"
DROP_PROPERTY_INDEX = "DROP PROPERTY INDEX IF EXISTS idx_vlabel_object_uuid;"

SQL_MIGRATIONS = [
    (CREATE_VLABEL_OBJECT, DROP_VLABEL_OBJECT),
    (CREATE_PROPERTY_INDEX, DROP_PROPERTY_INDEX),
]


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0004_auto_20211101_1848'),
    ]

    operations = [
            migrations.RunSQL(sql=forward, reverse_sql=reverse, hints={"target_db": "graph"})
            for forward, reverse in SQL_MIGRATIONS
    ]



from django.core.management.base import BaseCommand

from urlid_graph.utils import working
from urlid_graph.models import ObjectRepository


class Command(BaseCommand):
    help = "Update full-text search index"

    def add_arguments(self, parser):
        parser.add_argument(
            "--lock", action="store_true", default=False, help="Lock view for reading during refresh (faster)"
        )

    def handle(self, *args, **kwargs):
        lock = kwargs["lock"]
        concurrently = not lock

        lock_text = "(with lock)" if lock else "(without lock)"
        with working(f"Refreshing object materialized view {lock_text}"):
            ObjectRepository.objects.refresh(concurrently=concurrently)



from collections import defaultdict
from functools import lru_cache
from urllib.parse import urljoin

from cached_property import cached_property
from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.contrib.postgres.indexes import GinIndex
from django.contrib.postgres.search import SearchQuery, SearchRank, SearchVectorField
from django.core.exceptions import ObjectDoesNotExist
from django.db import connections, models
from django.db.utils import NotSupportedError
from django.template import Context, Template

from urlid_graph import settings as urlid_graph_settings

from . import formatting


def get_urlid_database_uri():
    db = settings.DATABASES[urlid_graph_settings.DJANGO_DATABASE]
    return f"postgres://{db['USER']}:{db['PASSWORD']}@{db['HOST']}:{db['PORT']}/{db['NAME']}"


@lru_cache(maxsize=64)
def get_template(template_code):
    return Template(template_code)


class EntityQuerySet(models.QuerySet):
    def by_name(self, names):
        return self.filter(name__in=[name.lower().strip() for name in names])


class BrasilIOEntityQuerySet(EntityQuerySet):
    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(
                base_url="https://id.brasil.io/",
                version=1,
            )
        )


class Entity(models.Model):
    brasilio = BrasilIOEntityQuerySet.as_manager()
    objects = EntityQuerySet.as_manager()

    uuid = models.UUIDField(primary_key=True)
    base_url = models.TextField(blank=False, null=False)
    name = models.TextField(blank=False, null=False)
    version = models.TextField(blank=False, null=False)

    @property
    def url(self):
        return urljoin(self.base_url, f"/{self.name}/v{self.version}/")

    @property
    def label_properties(self):
        return getattr(self.config, "label_properties", None) or []

    @property
    def label_template(self):
        return getattr(self.config, "label_template", None) or None

    @property
    def graph_node_conf(self):
        return getattr(self.config, "graph_node_conf", None) or {}

    @property
    def config(self):
        try:
            return EntityConfig.objects.for_entity(self.name)
        except ObjectDoesNotExist:
            return None

    @property
    def label(self):
        return getattr(self.config, "label", None) or self.name

    def __str__(self):
        return f"Entity {self.url}"


class ObjectMixin:
    """Mixin used by Object and ObjectRepository classes"""

    @property
    def url(self):
        return urljoin(self.entity.url, f"{self.internal_id}/")

    @property
    def label(self):
        label_template = self.entity.label_template
        if label_template is not None:
            return get_template(label_template).render(Context(self.properties))
        else:
            for prop_name in self.entity.label_properties:
                label = self.properties.get(prop_name)
                if label:
                    return label

        return f"{self.entity.name}: {self.internal_id}"

    @cached_property
    def properties(self):
        props = Property.objects.for_object(self).order_by("value_datetime")
        return {p.name: p.converted_value for p in props}

    def inject_properties(self, property_set):
        prop_key = "properties"
        if prop_key in self.__dict__:
            del self.__dict__[prop_key]
        self.__dict__[prop_key] = {p.name: p.converted_value for p in property_set}

    @cached_property
    def full_properties(self):
        db_props = Property.objects.for_object(self).order_by("-value_datetime")
        props = defaultdict(list)
        for prop in db_props:
            props[prop.name].append(
                {
                    "value": prop.value,
                    "value_type": prop.value_type,
                    "source": prop.source,
                    "value_datetime": prop.value_datetime.isoformat() if prop.value_datetime else None,
                }
            )
        return props

    def get_label_for_property(self, prop_name):
        try:
            config = EntityPropertyConfig.objects.for_object(self.entity.name, prop_name)
            return config.label
        except ObjectDoesNotExist:
            return prop_name

    def __str__(self):
        return f"Object {self.url}"


class Object(ObjectMixin, models.Model):
    # XXX: DO NOT USE! Use ObjectRepository instead

    uuid = models.UUIDField(primary_key=False, blank=False, null=False, db_index=True)
    entity = models.ForeignKey(
        Entity, on_delete=models.DO_NOTHING, db_column="entity_uuid", db_constraint=False, db_index=True
    )
    internal_id = models.TextField(blank=False, null=False)


class PropertyQuerySet(models.QuerySet):
    def for_object(self, obj):
        return self.filter(object_id=obj.uuid)

    def for_objects(self, objs):
        return self.filter(object_id__in=[obj.uuid for obj in objs])


class Property(models.Model):
    STR, DATE, DATETIME, FLOAT, INT, BOOL = range(1, 7)
    TYPE_CHOICES = [
        (STR, "str"),
        (DATE, "date"),
        (DATETIME, "datetime"),
        (FLOAT, "float"),
        (INT, "int"),
        (BOOL, "bool"),
    ]
    objects = PropertyQuerySet.as_manager()

    object = models.ForeignKey(
        "ObjectRepository",
        on_delete=models.DO_NOTHING,
        db_column="object_uuid",
        db_constraint=False,
        db_index=False,
    )
    name = models.TextField(blank=False, null=False)
    value = models.TextField(blank=True, null=True)
    value_type = models.SmallIntegerField(blank=False, null=False, choices=TYPE_CHOICES)
    source = models.TextField(blank=False, null=False)
    value_datetime = models.DateTimeField(blank=False, null=True)
    updated_at = models.DateTimeField(blank=True, null=True)

    class Meta:
        indexes = [
            models.Index(fields=["object", "value_type"]),
        ]

    def __str__(self):
        return f"Property {self.name} of {self.object.uuid}"

    @property
    def converted_value(self):
        value = (self.value or "").strip()
        if not value:
            return None

        property_convert_function = {
            Property.STR: str,
            Property.DATE: formatting.convert_date,
            Property.DATETIME: formatting.convert_datetime,
            Property.FLOAT: float,
            Property.INT: int,
            Property.BOOL: formatting.convert_bool,
        }
        return property_convert_function[self.value_type](value)


class EntityConfigManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(config_type=ElementConfig.ENTITY_CONFIG)

    @lru_cache()
    def for_entity(self, entity_name):
        return self.get_queryset().get(name=entity_name)


class RelationshipConfigManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(config_type=ElementConfig.REL_CONFIG)

    @lru_cache()
    def get_by_name(self, relationship_name):
        return self.get_queryset().get(name=relationship_name)


class EntityPropertyConfigManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.filter(config_type=ElementConfig.PROPERTY_CONFIG, parent_type=ElementConfig.ENTITY_CONFIG)

    @lru_cache()
    def for_object(self, parent_name, prop_name):
        return self.get_queryset().get(parent_name=parent_name, name=prop_name)


class RelPropertyConfigManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.filter(config_type=ElementConfig.PROPERTY_CONFIG, parent_type=ElementConfig.REL_CONFIG)


class StatsConfigManager(models.Manager):
    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.filter(config_type=ElementConfig.STATS_CONFIG)

    def get_stats(self):
        return self.get(name="stats")


class ElementConfig(models.Model):
    ENTITY_CONFIG, REL_CONFIG, PROPERTY_CONFIG, STATS_CONFIG = 1, 2, 3, 4
    TYPE_CHOICES = [
        (ENTITY_CONFIG, "entity"),
        (REL_CONFIG, "relationship"),
        (PROPERTY_CONFIG, "property"),
        (STATS_CONFIG, "stats"),
    ]
    PARENT_TYPE_CHOICES = [
        (ENTITY_CONFIG, "entity"),
        (REL_CONFIG, "relationship"),
    ]

    config_type = models.PositiveSmallIntegerField(choices=TYPE_CHOICES)
    name = models.TextField()
    parent_type = models.PositiveSmallIntegerField(choices=PARENT_TYPE_CHOICES, null=True)
    parent_name = models.TextField(null=True)
    label = models.TextField()
    data = models.JSONField()

    class Meta:
        unique_together = ["config_type", "name", "parent_type", "parent_name"]


class EntityConfig(ElementConfig):
    objects = EntityConfigManager()

    class Meta:
        proxy = True

    @property
    def label_properties(self):
        return self.data.get("label_properties", [])

    @property
    def label_template(self):
        return self.data.get("label_template", None)

    @property
    def graph_node_conf(self):
        return self.data.get("graph_node_conf", {})


class RelationshipConfig(ElementConfig):
    objects = RelationshipConfigManager()

    class Meta:
        proxy = True

    @property
    def from_graph_node_conf(self):
        return self.data.get("from_graph_node_conf", {})

    @property
    def to_graph_node_conf(self):
        return self.data.get("to_graph_node_conf", {})


class EntityPropertyConfig(ElementConfig):
    objects = EntityPropertyConfigManager()

    class Meta:
        proxy = True


class RelPropertyConfig(ElementConfig):
    objects = RelPropertyConfigManager()

    class Meta:
        proxy = True


class StatsConfig(ElementConfig):
    objects = StatsConfigManager()

    class Meta:
        proxy = True


class ObjectRepositoryQuerySet(models.QuerySet):
    def from_uuids(self, uuids):
        return self.filter(uuid__in=uuids).select_related("entity")

    def with_properties(self):
        props = Property.objects.for_objects(self.all()).order_by("value_datetime")

        qs = []
        for obj in self.all():
            objs_props = [p for p in props if p.object_id == obj.uuid]
            obj.inject_properties(objs_props)
            qs.append(obj)
        return qs

    def refresh(self, concurrently=False):
        """Refresh the materialized view in which this model is based on"""

        view_name = "urlid_graph_object_fts"
        regular_query = f"REFRESH MATERIALIZED VIEW {view_name}"
        concurrently_query = f"REFRESH MATERIALIZED VIEW CONCURRENTLY {view_name}"

        with connections[urlid_graph_settings.DJANGO_DATABASE].cursor() as cursor:
            if not concurrently:
                cursor.execute(regular_query)
            else:
                try:
                    cursor.execute(concurrently_query)
                except NotSupportedError:
                    # Materialized view not populated, so CONCURRENTLY cannot
                    # be used
                    cursor.execute(regular_query)

    def search(self, search_query, config=urlid_graph_settings.SEARCH_LANGUAGE):
        """Full-Text Search in object's search_data"""

        qs = self.filter(internal_id__isnull=False, entity__isnull=False)
        search_query = search_query or ""
        if not search_query:
            return qs
        words = search_query.split()
        query = None
        done = []  # to preserve order
        for word in words:
            if not word or word in done:
                continue
            if query is None:
                query = SearchQuery(word, config=config)
            else:
                query = query & SearchQuery(word, config=config)
            done.append(word)
        qs = qs.annotate(search_rank=SearchRank(models.F("search_data"), query)).filter(search_data=query)
        # Using `qs.query.add_ordering` will APPEND ordering fields instead
        # of OVERWRITTING (`qs.order_by` will overwrite).
        qs.query.add_ordering("-search_rank")
        return qs

    def search_many_entities(self, query, entities=all, limit_per_entity=10):
        """Search BrasilIO objects in more than one entity, by name

        Prefer to use this method since it also filter objects by entity (more performance)"""

        if entities is all:
            entities = Entity.brasilio.all()
        else:
            entities = Entity.brasilio.by_name(entities)

        results = []
        for entity in entities:
            qs = self.filter(entity=entity).search(query).select_related("entity")
            results.extend(qs[:limit_per_entity])
        results.sort(key=lambda row: row.search_rank, reverse=True)
        return results


class ObjectRepository(ObjectMixin, models.Model):
    objects = ObjectRepositoryQuerySet.as_manager()

    uuid = models.UUIDField(primary_key=True)
    entity = models.ForeignKey(
        Entity, on_delete=models.DO_NOTHING, db_column="entity_uuid", db_constraint=False, db_index=False
    )
    internal_id = models.TextField(blank=False, null=False)
    search_data = SearchVectorField(null=True)

    class Meta:
        db_table = "urlid_graph_object_repository"
        managed = False
        indexes = [GinIndex(fields=["search_data"])]
        unique_together = ["entity", "uuid", "internal_id"]


class SavedGraph(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    name = models.CharField(max_length=255)
    edges = ArrayField(models.CharField(max_length=255))
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"{self.user.username} - {self.name}"

    class Meta:
        unique_together = (("user", "name"),)



import uuid

from django.core.management.base import BaseCommand

from urlid_graph.models import Entity


class Command(BaseCommand):
    help = "Create Brasil.IO entities"

    def handle(self, *args, **options):
        brasilio_base_url = "https://id.brasil.io/"
        version = 1
        for name in ("company", "person", "candidacy", "court-case"):
            url = f"{brasilio_base_url}{name}/v{version}/"
            row = {
                "base_url": brasilio_base_url,
                "name": name,
                "version": version,
                "uuid": uuid.uuid5(uuid.NAMESPACE_URL, url),
            }
            obj = Entity.objects.filter(**row).first()
            created = False
            if obj is None:
                obj = Entity(**row)
                obj.save()
                created = True
            print(f"{'CREATED' if created else 'ALREADY EXISTS'} {name}, uuid = {obj.uuid}")



from pathlib import Path

from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Import Objects, Properties and Relationships from a directory"

    def add_arguments(self, parser):
        parser.add_argument("path")
        parser.add_argument("--filetype", action="append")

    def handle(self, *args, **options):
        path = Path(options["path"])
        file_types = options["filetype"]
        if not file_types:
            file_types = all
        else:
            file_types = [file_type.lower() for file_type in file_types]
            for file_type in file_types:
                if file_type not in ("property", "object", "relationship"):
                    raise ValueError(f"Unknown value '{file_type}'")

        # First, import properties (before objects - so when objects are
        # imported we'll fill search_data with property data)
        if file_types is all or "property" in file_types:
            filenames = path.glob("*property*.csv*")
            filenames = [str(filename.absolute()) for filename in filenames]
            ok = call_command("import_properties", *filenames) == "True"
            if not ok:
                return ok

        # Second, import objects
        if file_types is all or "object" in file_types:
            filenames = path.glob("*object*.csv*")
            filenames = [str(filename.absolute()) for filename in filenames]
            ok = call_command("import_objects", *filenames) == "True"
            if not ok:
                return ok

        # Finally, import relations
        if file_types is all or "relationship" in file_types:
            relations_filenames = []
            for filename in path.glob("*relationship*.csv*"):
                relationship = filename.absolute().name.split("relationship")[1].split(".csv")[0]
                filename = str(filename.absolute())
                if relationship.startswith("-"):
                    relationship = relationship[1:]
                if relationship.endswith("-"):
                    relationship = relationship[:-1]
                relations_filenames.append(f"{relationship}--{filename}")

            ok = call_command("import_relations", *relations_filenames) == "True"
            if not ok:
                return ok



# Generated by Django 3.2.9 on 2021-11-21 04:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0005_auto_20211104_1245'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='objectrepository',
            table='urlid_graph_object_repository',
        ),
    ]



from collections import OrderedDict

import rows

from urlid_graph.commands import ImportCommand
from urlid_graph.models import Object


class Command(ImportCommand):
    schema = OrderedDict(
        [
            ("entity_uuid", rows.fields.UUIDField),
            ("internal_id", rows.fields.TextField),
            ("uuid", rows.fields.UUIDField),
        ]
    )
    table_name = Object._meta.db_table



# Generated by Django 3.2 on 2021-10-27 14:03

from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('urlid_graph', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SavedGraph',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('edges', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), size=None)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'name')},
            },
        ),
    ]



from collections import OrderedDict

import rows

from urlid_graph.commands import ImportCommand
from urlid_graph.models import Property


class Command(ImportCommand):
    schema = OrderedDict(
        [
            ("id", rows.fields.IntegerField),
            ("object_uuid", rows.fields.UUIDField),
            ("value_type", rows.fields.IntegerField),
            ("name", rows.fields.TextField),
            ("value", rows.fields.TextField),
            ("source", rows.fields.TextField),
            ("value_datetime", rows.fields.DatetimeField),
            ("updated_at", rows.fields.DatetimeField),
        ]
    )
    table_name = Property._meta.db_table
